#include <stdio.h>
#include <iostream>
using namespace std;

//
// Estructur del nodo de la lista
struct NodoD {
    int elemento;
    NodoD *sig, *ant;

    NodoD(int data, NodoD* nextPtr = NULL, NodoD* antPtr = NULL): elemento(data),sig(nextPtr), ant(antPtr){}
};

class listaD {
private:
    NodoD *cab, *cola, *actual; // Apuntadores a la cabeza, la cola y el iterador actual
	int longitud;
public:
    listaD();
    listaD(listaD& otralistaD);
    ~listaD();

    bool insertarCabeza(int element);
    bool insertarCola(int element);

    bool sacarInicio();
    bool sacarFinal();
    bool eliminar(int pos);
    bool eliminarlista();

    NodoD* inicio();
    NodoD* fin();
    NodoD* siguienteNodo();
    NodoD* anteriorNodo();
    NodoD* actualNodo() const;

    int longitudlista();
    void listarElementos();
    void listarElementosI();
};

//
// Constructor sin parmetros
listaD::listaD(){
  cab = cola = actual = NULL;

  longitud = 0;
};

//
// Constructor a partir de otra lista
listaD::listaD(listaD& otralistaD){
    cab = cola = actual = NULL;
    longitud = 0;

    for (NodoD* iptr=otralistaD.inicio(); iptr != NULL; iptr = otralistaD.siguienteNodo()) {
        insertarCola(iptr->elemento);
    }
}

//
// Destructor de lista, elimina todos los nodos y libera el espacio
listaD::~listaD(){

    while (cab != NULL) {
      actual = cab;
      cab = cab->sig;
      delete actual;
    }
    cola = NULL;
    longitud = 0;
}

//
// Inserta el elemento dado de primero en la lista
bool listaD::insertarCabeza(int element){
  	NodoD *np;

	np=new NodoD(element,NULL,NULL);
	if(cab == NULL)
		cab = cola = np;
	else{
		np->sig=cab;
		cab->ant = np;
		cab=np;
	}

	longitud++;
	return true;
};


//

// Inserta el elemento dado de ltimo en la lista
bool listaD::insertarCola(int element){
  	NodoD *np;

	np=new NodoD(element,NULL);
	if(cab==NULL)
		cab= cola = np;
	else {
        cola->sig = np;
        np->ant = cola;
		cola=np;
	}

    longitud++;
	return true;
};


//
// Elimina el primer elemento de la lista
// Devuelve falso si la lista est vacia
bool listaD::sacarInicio(){
    NodoD* ptr;

    if (cab != NULL) {
       ptr = cab;

        if (cab == cola) // Un solo elemento en la cola
            cab = cola = actual = NULL;
        else {
            cab->sig->ant = NULL;
            cab = cab->sig;
            actual = cab;
        }
        delete ptr;
        longitud--;
        return true;
    }
    return false;
}


//
// Elimina el ltimo elemento de la lista
// Devuelve falso si la lista est vacia
bool listaD::sacarFinal(){
    NodoD* ptr;

    if (cola != NULL) {
        ptr = cola;
        if (cab == cola)
                cab = cola = actual =  NULL;
        else {
           cola->ant->sig = NULL;
           cola = cola->ant;
           actual = cab;
        }
        delete ptr;
        longitud--;
        return true;
    }
    return false;
}

//
// Elimina un elemento dada su posici
// Devuelve falso si la posicin no existe
bool listaD::eliminar(int pos){
    NodoD *ptr = cab;
    int i= 1;

    if ( pos < 1 || pos > longitud)
        return false;


    while (i < pos && ptr != NULL ) {
        ptr = ptr->sig;
        i++;
    }

    if (ptr != NULL){
        if (ptr == cab && ptr == cola) { // Un solo elemento en la cola
           eliminarlista();
        }
        else if (ptr == cab) {  // Se elimina el primero
            sacarInicio();
        }
        else if (ptr == cola) // Se elimina el ultimo
            sacarFinal();
        else { // Se elimina un elemento intermedio
            ptr->sig->ant = ptr->ant;
            ptr->ant->sig = ptr->sig;
            delete ptr;
            longitud--;
        }
        return true;
    }
    return false;
}

//
// Elimina todos los elementos de la lista y crea la lista vacia
bool listaD::eliminarlista(){
    NodoD *ptr= cab;

    while (ptr != NULL){
        actual = ptr;
        ptr = ptr->sig;
        delete actual;
    }
    cab = cola = actual = NULL;
    longitud = 0;
}

//
// Devuelve el primer nodo de la lista y setea el iterador
NodoD* listaD::inicio(){
    return (actual = cab);
}

//
// Devuelve el ltimo elemento de la lista y setea el iterador
NodoD* listaD::fin(){
    return (actual = cola);
}

//
// Devuelve el siguiente nodo de la lista segn el iterador
NodoD* listaD::siguienteNodo(){
    if (actual != NULL)
        return (actual = actual->sig);
    else
        return NULL;
}

//
// Devuelve el nodo anterior de la lista segn el iterador
NodoD* listaD::anteriorNodo(){
    if (actual != NULL)
        return (actual = actual->ant);
    else
        return NULL;
}

//
// Devuelve el nodo actual de la lista segn el iterador
NodoD* listaD::actualNodo() const{
    return actual;
}

//
// Devuelve la longitud de la lista
int listaD::longitudlista(){
	return longitud;
};

//
// Imprime los elementos en el orden de la lista en el standard output
void listaD::listarElementos(){
	NodoD* iptr;

    cout << "Entre en listar " << longitud << endl;

    iptr = cab;
    while(iptr != NULL) {
        cout << "Elemento : " << iptr->elemento << endl;
        iptr = iptr->sig;
    }
	cout << endl;
}

//
// Imprime los elementos en el orden inverso de la lista en el standard output
void listaD::listarElementosI(){
	NodoD* iptr;

    cout << "Entre en listar invertidos " << longitud << endl;

    iptr = cola;
    while(iptr != NULL) {
        cout << "Elemento : " << iptr->elemento << endl;
        iptr = iptr->ant;
    }
	cout << endl;
}

main() {

    listaD L; int e;

	cout <<("\nImplementacion de listas Doblemente Enlazadas\n");
	L.insertarCabeza(2);
	L.insertarCabeza(4);
	L.insertarCabeza(3);
	L.insertarCabeza(1);
	L.insertarCabeza(5);
	L.insertarCabeza(3);
	L.insertarCabeza(0);
	L.listarElementos();

	listaD L2(L);

	cout << endl;
	cout << "Eliminar posicion 3" << endl;
	L.eliminar(3);
	L.listarElementos();

	cout << endl;
	cout << "Eliminar posicion 10" << endl;
	L.eliminar(10);
	L.listarElementos();

	cout << endl;
	cout << "Eliminar posicion 1" << endl;
	L.eliminar(1);
	L.listarElementos();

	cout << endl;
	cout << "Eliminar posicion 5" << endl;
	L.eliminar(5);
	L.listarElementos();

	cout << "listando L2" << endl;
	L2.listarElementos();

	cout << "Limpiando L" << endl;
	L.eliminarlista();
	L. listarElementos();

	cout << "listando L2" << endl;
	L2.listarElementos();

	cout << "listando L2 Invertido" << endl;
	L2.listarElementosI();

	cout << "Sacando El Inicio" << endl;
	L2.sacarInicio();
	L2.listarElementos();

	cout << "Sacando El Final" << endl;
	L2.sacarFinal();
	L2.listarElementos();
}

