#include <stdio.h>
#include <iostream>
using namespace std;

//
// Estructura del nodo almacenar en la lista
struct Nodo {
    int elemento;
    Nodo* sig;

    // Constructor del nodo
    Nodo(int data, Nodo* nextPtr = NULL): elemento(data),sig(nextPtr){}
};

class lista {
private:
    Nodo *cab, *cola, *actual; // Apuntador al primero, ltimo y el ietrador actual
	int longitud;
public:
    lista();
    lista(lista& otraLista);
    ~lista();

    bool insertarCabeza(int element);
    bool insertarCola(int element);

    bool sacarInicio();
    bool sacarFinal();
    bool eliminar(int pos);
    bool eliminarLista();

    Nodo* inicio();
    Nodo* fin();
    Nodo* siguienteNodo();
    Nodo* actualNodo() const;

    int longitudLista();
    void listarElementos();
};

//
// Constructor por defecto
lista::lista(){
  cab = cola = actual = NULL;

  longitud = 0;
};

//
// Constructor a partir de otra lista
lista::lista(lista& otraLista){
    cab = cola = actual = NULL;
    longitud = 0;

    for (Nodo* iptr=otraLista.inicio(); iptr != NULL; iptr = otraLista.siguienteNodo()) {
        insertarCola(iptr->elemento);
    }
}

//
// Destructor de la lista, libera el espacio de todos los nodos de la lista
//
lista::~lista(){

    while (cab != NULL) {
      actual = cab;
      cab = cab->sig;
      delete actual;
    }
    cola = NULL;
    longitud = 0;
}

//
// Inserta un nodo de primero en la lista
bool lista::insertarCabeza(int element){
  	Nodo *np;

	np=new Nodo(element,NULL);
	if(cab==NULL)
		cab= cola = np;
	else{
		np->sig=cab;
		cab=np;
	}

	longitud++;
	return true;
};


//
// Inserta un nodo de ltimo en la lista
bool lista::insertarCola(int element){
  	Nodo *np;

	np=new Nodo(element,NULL);
	if(cab==NULL)
		cab= cola = np;
	else {
        cola->sig = np;
		cola=np;
	}

    longitud++;
	return true;
};

// Elimina el primer nodo de la lista
// Devuelve falso si la lista est vacia
bool lista::sacarInicio(){
    Nodo* ptr;

    if (cab != NULL) {
       ptr = cab;

        if (cab == cola) // Un solo elemento en la cola
            cab = cola = actual = NULL;
        else {
            cab = cab->sig;
            actual = cab;
        }
        delete ptr;
        longitud--;
        return true;
    }
    return false;
}

//
// Elimina el ltimo elemento de la lista
// Devuelve falso si la lista est vacia
bool lista::sacarFinal(){
    Nodo* ptr;

    if (cola != NULL) {
        ptr = cola;
        if (cab == cola)
                cab = cola = actual =  NULL;
        else {
           actual = cab;
           while (actual->sig != cola)
                actual = actual->sig;
           actual->sig = NULL;
           cola = actual;
        }
        delete ptr;
        longitud--;
        return true;
    }
    return false;
}

// Elimina un nodo dada su posicin
// Devuelve falso si la posicin no existe
bool lista::eliminar(int pos){
    Nodo *ptr = cab, *ptrPrevio = cab;
    int i= 1;

    if ( pos < 1 || pos > longitud)
        return false;


    while (i < pos && ptr != NULL ) {
        ptr = ptr->sig;
        ptrPrevio = (i==1?ptrPrevio:ptrPrevio->sig);
        i++;
    }

    if (ptr != NULL){
        if (ptr == cab && ptr == cola) { // Un solo elemento en la cola
           cab = cola = actual = NULL;
        }
        else if (ptr == cab) {  // Se elimina el primero
            cab = cab->sig;
        }
        else if (ptr == cola){ // Se elimina el ultimo
            cola = ptrPrevio;
            ptrPrevio->sig = NULL;
        } else { // Se elimina un elemento intermedio
            ptrPrevio->sig = ptr->sig;
        }

        delete ptr;
        longitud--;
        return true;
    }
    return false;
}

//
// Elimina completamente la lista y crea la lista vacia
bool lista::eliminarLista(){
    Nodo *ptr= cab;

    while (ptr != NULL){
        actual = ptr;
        ptr = ptr->sig;
        delete actual;
    }
    cab = cola = actual = NULL;
    longitud = 0;
}

//
// Devuelve el primer nodo de la lista y setea el iterador
Nodo* lista::inicio(){
    return (actual = cab);
}

//
// Devuelve el ltimo nodo de la lista y setea el iterador
Nodo* lista::fin(){
    return (actual = cola);
}

//
// Devuelve el siguiente nodo de la lista y seta el iterador
Nodo* lista::siguienteNodo(){
    return (actual = actual->sig);
}

//
// Devuelve el nodo actual del iterador
Nodo* lista::actualNodo() const{
    return actual;
}

//
// Devuelve la longitud de la lista
int lista::longitudLista(){
	return longitud;
};

//
// Lista los elementos de la lista en el standard output
void lista::listarElementos(){
	Nodo* iptr;

    cout << "Entre en listar " << longitud << endl;

    iptr = cab;
    while(iptr != NULL) {
        cout << "Elemento : " << iptr->elemento << endl;
        iptr = iptr->sig;
    }
	cout << endl;
}


main() {

    lista L; int e;

	cout <<("\nImplementacion de Listas Enlazadas\n");
	L.insertarCabeza(2);
	L.insertarCabeza(4);
	L.insertarCabeza(3);
	L.insertarCabeza(1);
	L.insertarCabeza(5);
	L.insertarCabeza(3);
	L.insertarCabeza(0);
	L.listarElementos();

	lista L2(L);

	cout << endl;
	cout << "Eliminar posicion 3" << endl;
	L.eliminar(3);
	L.listarElementos();

	cout << endl;
	cout << "Eliminar posicion 10" << endl;
	L.eliminar(10);
	L.listarElementos();

	cout << endl;
	cout << "Eliminar posicion 1" << endl;
	L.eliminar(1);
	L.listarElementos();

	cout << endl;
	cout << "Eliminar posicion 5" << endl;
	L.eliminar(5);
	L.listarElementos();

	cout << "Listando L2" << endl;
	L2.listarElementos();

	cout << "Limpiando L" << endl;
	L.eliminarLista();
	L. listarElementos();

	cout << "Listando L2" << endl;
	L2.listarElementos();

	cout << "Sacando El Inicio" << endl;
	L2.sacarInicio();
	L2.listarElementos();

	cout << "Sacando El Final" << endl;
	L2.sacarFinal();
	L2.listarElementos();
}

